@@ -5,7 +5,8 @@
     maybe_uninit_array_assume_init,
     once_cell,
     io_safety,
-    linked_list_cursors
+    linked_list_cursors,
+    vec_into_raw_parts
 )]
 #![warn(
     unsafe_op_in_unsafe_fn,
@@ -19,6 +20,7 @@
     clippy::inconsistent_struct_constructor,
     clippy::wrong_self_convention,
     clippy::missing_const_for_fn,
+    clippy::new_without_default,
     rustdoc::broken_intra_doc_links,
     rustdoc::private_intra_doc_links
 )]
@@ -35,14 +37,8 @@
 mod syringe;
 pub use syringe::*;
 
-mod process;
-pub use process::*;
-
-mod process_ref;
-pub use process_ref::*;
-
-mod process_module;
-pub use process_module::*;
+/// Module containing process abstractions and utilities.
+pub mod process;
 
 #[cfg(any(feature = "remote_procedure", feature = "doc_cfg"))]
 #[cfg_attr(feature = "doc_cfg", doc(cfg(feature = "remote_procedure")))]
@@ -50,16 +46,6 @@ mod remote_procedure;
 #[cfg(any(feature = "remote_procedure", feature = "doc_cfg"))]
 pub use remote_procedure::*;
 
-#[cfg_attr(not(feature = "process_memory"), allow(dead_code))]
-#[cfg(any(feature = "process_memory", feature = "doc_cfg"))]
-#[cfg_attr(feature = "doc_cfg", doc(cfg(feature = "process_memory")))]
-/// Module containing utilities for dealing with memory of another process.
-pub mod process_memory;
-#[cfg_attr(not(feature = "process_memory"), allow(dead_code))]
-#[cfg(not(any(feature = "process_memory", feature = "doc_cfg")))]
-/// Module containing utilities for dealing with memory of another process.
-pub(crate) mod process_memory;
-
 pub(crate) mod utils;
 
 /// Module containing the error enums used in this crate.
