@@ -1,25 +1,25 @@
 use cstr::cstr;
-use get_last_error::Win32Error;
 use iced_x86::{code_asm::*, IcedError};
 use num_enum::TryFromPrimitive;
 use path_absolutize::Absolutize;
-use std::{io, ffi::c_void, lazy::OnceCell, mem, path::Path};
+use std::{ffi::c_void, io, lazy::OnceCell, mem, path::Path};
 use u16cstr::u16cstr;
 use widestring::U16CString;
 use winapi::shared::{
-    minwindef::{BOOL, DWORD, HMODULE, FALSE},
+    minwindef::{BOOL, DWORD, FALSE, HMODULE},
     ntdef::LPCWSTR,
 };
 
 use crate::{
     error::{ExceptionCode, SyringeError},
-    process_memory::{RemoteBox, RemoteBoxAllocator},
-    ModuleHandle, ProcessModule, ProcessRef,
+    process::{
+        memory::{RemoteBox, RemoteBoxAllocator},
+        BorrowedProcess, BorrowedProcessModule, ModuleHandle, OwnedProcess, Process, ProcessModule,
+    },
 };
 
 #[cfg(all(target_arch = "x86_64", feature = "into_x86_from_x64"))]
 use {
-    crate::utils::retry_with_filter,
     goblin::pe::PE,
     std::{convert::TryInto, fs, mem::MaybeUninit, path::PathBuf, time::Duration},
     widestring::U16Str,
@@ -63,18 +63,17 @@ impl InjectHelpData {
     }
 }
 
-/// An injector that can inject modules (.dll's) into processes.
-/// This struct keeps internal state to allow for faster injcetions if reused.
+/// An injector that can inject modules (.dll's) into a target process.
 ///
 /// # Example
 /// ```no_run
-/// use dll_syringe::{Syringe, Process};
+/// use dll_syringe::{Syringe, process::OwnedProcess};
 ///
 /// // find target process by name
-/// let target_process = Process::find_first_by_name("target_process").unwrap();
+/// let target_process = OwnedProcess::find_first_by_name("target_process").unwrap();
 ///
 /// // create a new syringe for the target process
-/// let mut syringe = Syringe::for_process(&target_process);
+/// let mut syringe = Syringe::for_process(target_process);
 ///
 /// // inject the payload into the target process
 /// let injected_payload = syringe.inject("injection_payload.dll").unwrap();
@@ -85,86 +84,114 @@ impl InjectHelpData {
 /// syringe.eject(injected_payload).unwrap();
 /// ```
 #[derive(Debug)]
-pub struct Syringe<'a> {
-    pub(crate) process: ProcessRef<'a>,
+pub struct Syringe {
     pub(crate) inject_help_data: OnceCell<InjectHelpData>,
-    pub(crate) remote_allocator: RemoteBoxAllocator<'a>,
-    load_library_w_stub: OnceCell<LoadLibraryWStub<'a>>,
+    pub(crate) remote_allocator: RemoteBoxAllocator,
+    load_library_w_stub: OnceCell<LoadLibraryWStub>,
     #[cfg(feature = "remote_procedure")]
     pub(crate) get_proc_address_stub:
-        OnceCell<crate::RemoteProcedureStub<'a, crate::GetProcAddressParams, FARPROC>>,
+        OnceCell<crate::RemoteProcedureStub<crate::GetProcAddressParams, FARPROC>>,
 }
 
-impl<'a> Syringe<'a> {
-    /// Creates a new syringe for the given process.
-    pub fn for_process(process: impl Into<ProcessRef<'a>>) -> Self {
-        let process = process.into();
+impl Syringe {
+    /// Creates a new syringe for the given target process.
+    #[must_use]
+    pub fn for_process(process: OwnedProcess) -> Self {
         Self {
-            process,
-            inject_help_data: OnceCell::new(),
             remote_allocator: RemoteBoxAllocator::new(process),
+            inject_help_data: OnceCell::new(),
             load_library_w_stub: OnceCell::new(),
             #[cfg(feature = "remote_procedure")]
             get_proc_address_stub: OnceCell::new(),
         }
     }
 
-    /// Inject the module at the given path into the target process.
+    /// Returns the target process for this syringe.
+    pub fn process(&self) -> BorrowedProcess<'_> {
+        self.remote_allocator.process()
+    }
+
+    /// Injects the module from the given path into the target process.
     ///
     /// # Limitations
     /// - The target process and the given module need to be of the same bitness.
     /// - If the current process is `x64` the target process can be either `x64` (always available) or `x86` (with the `into_x86_from_x64` feature enabled).
     /// - If the current process is `x86` the target process can only be `x86`.
     pub fn inject(
-        &mut self,
+        &self,
         payload_path: impl AsRef<Path>,
-    ) -> Result<ProcessModule<'a>, SyringeError> {
-        self.load_library_w_stub.get_or_try_init(|| {
+    ) -> Result<BorrowedProcessModule<'_>, SyringeError> {
+        let load_library_w = self.load_library_w_stub.get_or_try_init(|| {
             let inject_data = self
                 .inject_help_data
-                .get_or_try_init(|| Self::load_inject_help_data_for_process(self.process))?;
-            LoadLibraryWStub::build(inject_data, &mut self.remote_allocator)
+                .get_or_try_init(|| Self::load_inject_help_data_for_process(self.process()))?;
+            LoadLibraryWStub::build(inject_data, &self.remote_allocator)
         })?;
-        let load_library_w = self.load_library_w_stub.get_mut().unwrap();
 
         let module_path = payload_path.as_ref().absolutize()?;
         let wide_module_path =
             U16CString::from_os_str(module_path.as_os_str())?.into_vec_with_nul();
-        let mut remote_wide_module_path = self
+        let remote_wide_module_path = self
             .remote_allocator
             .alloc_and_copy(wide_module_path.as_slice())?;
 
-        let injected_module_handle = load_library_w.call(remote_wide_module_path.as_mut_ptr())?;
-        let injected_module = unsafe { ProcessModule::new_unchecked(injected_module_handle, self.process) };
+        let injected_module_handle =
+            load_library_w.call(remote_wide_module_path.as_raw_ptr().cast())?;
+        let injected_module =
+            unsafe { ProcessModule::new_unchecked(injected_module_handle, self.process()) };
 
         debug_assert_eq!(
-            injected_module,
-            self.process.find_module_by_path(module_path)?.unwrap()
+            Some(injected_module),
+            self.process().find_module_by_path(module_path)?
         );
 
         Ok(injected_module)
     }
 
-    /// Ejects a previously injected module from its target process.
-    pub fn eject(&self, module: ProcessModule<'_>) -> Result<(), SyringeError> {
+    /// Injects the module from the given path into the target process, if it is not already loaded.
+    ///
+    /// # Limitations
+    /// - The target process and the given module need to be of the same bitness.
+    /// - If the current process is `x64` the target process can be either `x64` (always available) or `x86` (with the `into_x86_from_x64` feature enabled).
+    /// - If the current process is `x86` the target process can only be `x86`.
+    pub fn find_or_inject(
+        &self,
+        payload_path: impl AsRef<Path>,
+    ) -> Result<BorrowedProcessModule<'_>, SyringeError> {
+        let payload_path = payload_path.as_ref();
+        match self.process().find_module_by_path(payload_path) {
+            Ok(Some(module)) => Ok(module),
+            Ok(None) => self.inject(payload_path),
+            Err(err) => Err(err.into()),
+        }
+    }
+
+    /// Ejects a module from the target process.
+    ///
+    /// # Panics
+    /// This method panics if the given module was not loaded in the target process.
+    pub fn eject(&self, module: BorrowedProcessModule<'_>) -> Result<(), SyringeError> {
         assert!(
-            module.process() == self.process,
+            module.process() == &self.process(),
             "trying to eject a module from a different process"
         );
 
         let inject_data = self
             .inject_help_data
-            .get_or_try_init(|| Self::load_inject_help_data_for_process(self.process))?;
+            .get_or_try_init(|| Self::load_inject_help_data_for_process(self.process()))?;
 
-        let exit_code = self.process.run_remote_thread(
+        let exit_code = self.process().run_remote_thread(
             unsafe { mem::transmute(inject_data.get_free_library_fn_ptr()) },
-            module.handle().cast(),
+            module.handle(),
         )?;
 
         let free_library_result = exit_code as BOOL;
 
         if free_library_result == FALSE {
-            return Err(SyringeError::RemoteIo(io::Error::new(io::ErrorKind::Other, "failed to eject module from process")));
+            return Err(SyringeError::RemoteIo(io::Error::new(
+                io::ErrorKind::Other,
+                "failed to eject module from process",
+            )));
         }
         if let Ok(exception) = ExceptionCode::try_from_primitive(exit_code) {
             return Err(SyringeError::RemoteException(exception));
@@ -172,10 +199,10 @@ impl<'a> Syringe<'a> {
 
         debug_assert!(
             !self
-                .process
+                .remote_allocator
+                .process()
                 .module_handles()?
-                .as_ref()
-                .contains(&module.handle()),
+                .any(|m| m == module.handle()),
             "ejected module survived"
         );
 
@@ -183,7 +210,7 @@ impl<'a> Syringe<'a> {
     }
 
     pub(crate) fn load_inject_help_data_for_process(
-        process: ProcessRef<'_>,
+        process: BorrowedProcess<'_>,
     ) -> Result<InjectHelpData, SyringeError> {
         let is_target_x64 = process.is_x64()?;
         let is_self_x64 = cfg!(target_arch = "x86_64");
@@ -198,36 +225,47 @@ impl<'a> Syringe<'a> {
 
     pub(crate) fn remote_exit_code_to_exception(exit_code: u32) -> Result<(), SyringeError> {
         if exit_code == 0 {
-            return Ok(())
+            return Ok(());
         }
 
         match ExceptionCode::try_from_primitive(exit_code) {
             Ok(exception) => Err(SyringeError::RemoteException(exception)),
-            Err(_) => Err(SyringeError::RemoteIo(io::Error::new(io::ErrorKind::Other, "unknown remote process error"))),
+            Err(_) => Err(SyringeError::RemoteIo(io::Error::new(
+                io::ErrorKind::Other,
+                "unknown remote process error",
+            ))),
         }
     }
 
-    pub(crate) fn remote_exit_code_to_error_or_exception(exit_code: u32) -> Result<(), SyringeError> {
+    pub(crate) fn remote_exit_code_to_error_or_exception(
+        exit_code: u32,
+    ) -> Result<(), SyringeError> {
         if exit_code == 0 {
-            return Ok(())
+            return Ok(());
         }
 
         match ExceptionCode::try_from_primitive(exit_code) {
             Ok(exception) => Err(SyringeError::RemoteException(exception)),
-            Err(_) => Err(SyringeError::RemoteIo(Win32Error::new(exit_code).into())),
+            Err(_) => Err(SyringeError::RemoteIo(io::Error::from_raw_os_error(
+                exit_code as _,
+            ))),
         }
     }
 
     fn load_inject_help_data_for_current_target() -> Result<InjectHelpData, SyringeError> {
         let kernel32_module =
-            ProcessModule::__find_local_by_name_or_abs_path(u16cstr!("kernel32.dll"))?.unwrap();
+            BorrowedProcessModule::find_local_by_name_or_abs_path_wstr(u16cstr!("kernel32.dll"))?
+                .unwrap();
 
-        let load_library_fn_ptr = kernel32_module.__get_local_procedure(cstr!("LoadLibraryW"))?;
-        let free_library_fn_ptr = kernel32_module.__get_local_procedure(cstr!("FreeLibrary"))?;
-        let get_last_error_fn_ptr = kernel32_module.__get_local_procedure(cstr!("GetLastError"))?;
+        let load_library_fn_ptr =
+            kernel32_module.get_local_procedure_address_cstr(cstr!("LoadLibraryW"))?;
+        let free_library_fn_ptr =
+            kernel32_module.get_local_procedure_address_cstr(cstr!("FreeLibrary"))?;
+        let get_last_error_fn_ptr =
+            kernel32_module.get_local_procedure_address_cstr(cstr!("GetLastError"))?;
         #[cfg(feature = "remote_procedure")]
         let get_proc_address_fn_ptr =
-            kernel32_module.__get_local_procedure(cstr!("GetProcAddress"))?;
+            kernel32_module.get_local_procedure_address_cstr(cstr!("GetProcAddress"))?;
 
         Ok(InjectHelpData {
             kernel32_module: kernel32_module.handle(),
@@ -244,15 +282,12 @@ impl<'a> Syringe<'a> {
     #[cfg(target_arch = "x86_64")]
     #[cfg(feature = "into_x86_from_x64")]
     fn _load_inject_help_data_for_process(
-        process: ProcessRef<'_>,
+        process: BorrowedProcess<'_>,
     ) -> Result<InjectHelpData, SyringeError> {
         // get kernel32 handle of target process (may fail if target process is currently starting and has not loaded kernel32 yet)
-        let kernel32_module = retry_with_filter(
-            || process.find_module_by_name("kernel32.dll"),
-            Option::is_some,
-            Duration::from_secs(1),
-        )?
-        .unwrap();
+        let kernel32_module = process
+            .wait_for_module_by_name("kernel32.dll", Duration::from_secs(1))?
+            .unwrap();
 
         // get path of kernel32 used in target process
         let kernel32_path = if process.is_x86()? {
@@ -303,12 +338,12 @@ impl<'a> Syringe<'a> {
     }
 
     #[cfg(all(target_arch = "x86_64", feature = "into_x86_from_x64"))]
-    fn wow64_dir() -> Result<PathBuf, Win32Error> {
+    fn wow64_dir() -> Result<PathBuf, io::Error> {
         let mut path_buf = MaybeUninit::uninit_array::<MAX_PATH>();
         let path_buf_len: u32 = path_buf.len().try_into().unwrap();
         let result = unsafe { GetSystemWow64DirectoryW(path_buf[0].as_mut_ptr(), path_buf_len) };
         if result == 0 {
-            return Err(Win32Error::get_last_error());
+            return Err(io::Error::last_os_error());
         }
 
         let path_len = result as usize;
@@ -318,29 +353,29 @@ impl<'a> Syringe<'a> {
 }
 
 #[derive(Debug)]
-struct LoadLibraryWStub<'a> {
-    code: RemoteBox<'a, [u8]>,
-    result: RemoteBox<'a, ModuleHandle>,
+struct LoadLibraryWStub {
+    code: RemoteBox<[u8]>,
+    result: RemoteBox<ModuleHandle>,
 }
 
-impl<'a> LoadLibraryWStub<'a> {
+impl LoadLibraryWStub {
     fn build(
         inject_data: &InjectHelpData,
-        remote_allocator: &mut RemoteBoxAllocator<'a>,
+        remote_allocator: &RemoteBoxAllocator,
     ) -> Result<Self, SyringeError> {
-        let mut result = remote_allocator.alloc_uninit::<ModuleHandle>()?;
+        let result = remote_allocator.alloc_uninit::<ModuleHandle>()?;
 
         let code = if remote_allocator.process().is_x86()? {
             Self::build_code_x86(
                 inject_data.get_load_library_fn_ptr() as *const _,
-                result.as_mut_ptr().cast(),
+                result.as_raw_ptr().cast(),
                 inject_data.get_get_last_error() as *const _,
             )
             .unwrap()
         } else {
             Self::build_code_x64(
                 inject_data.get_load_library_fn_ptr() as *const _,
-                result.as_mut_ptr().cast(),
+                result.as_raw_ptr().cast(),
                 inject_data.get_get_last_error() as *const _,
             )
             .unwrap()
@@ -350,11 +385,11 @@ impl<'a> LoadLibraryWStub<'a> {
         Ok(Self { code, result })
     }
 
-    fn call(&mut self, remote_wide_module_path: *mut u16) -> Result<ModuleHandle, SyringeError> {
+    fn call(&self, remote_wide_module_path: *mut u16) -> Result<ModuleHandle, SyringeError> {
         // creating a thread that will call LoadLibraryW with a pointer to payload_path as argument
-        let exit_code = self.process().run_remote_thread(
-            unsafe { mem::transmute(self.code.as_ptr()) },
-            remote_wide_module_path.cast(),
+        let exit_code = self.code.process().run_remote_thread(
+            unsafe { mem::transmute(self.code.as_raw_ptr()) },
+            remote_wide_module_path,
         )?;
 
         Syringe::remote_exit_code_to_error_or_exception(exit_code)?;
@@ -365,7 +400,8 @@ impl<'a> LoadLibraryWStub<'a> {
         Ok(injected_module_handle)
     }
 
-    fn process(&self) -> ProcessRef<'a> {
+    #[allow(dead_code)]
+    fn process(&self) -> BorrowedProcess<'_> {
         self.code.process()
     }
 
@@ -399,7 +435,11 @@ impl<'a> LoadLibraryWStub<'a> {
         asm.ret_1(4)?; // Restore stack ptr. (Callee cleanup)
 
         let code = asm.assemble(0x1234_5678)?;
-        debug_assert_eq!(code, asm.assemble(0x1111_2222)?, "LoadLibraryW x86 stub is not location independent");
+        debug_assert_eq!(
+            code,
+            asm.assemble(0x1111_2222)?,
+            "LoadLibraryW x86 stub is not location independent"
+        );
 
         Ok(code)
     }
@@ -434,7 +474,11 @@ impl<'a> LoadLibraryWStub<'a> {
         asm.ret()?; // Restore stack ptr. (Callee cleanup)
 
         let code = asm.assemble(0x1234_5678)?;
-        debug_assert_eq!(code, asm.assemble(0x1111_2222)?, "LoadLibraryW x64 stub is not location independent");
+        debug_assert_eq!(
+            code,
+            asm.assemble(0x1111_2222)?,
+            "LoadLibraryW x64 stub is not location independent"
+        );
 
         Ok(code)
     }
